//
//  FieldViewController.m
//  Field
//
//  Created by Michael Heyeck on 3/5/10.
//  Copyright Fair Oaks Labs, Inc. 2010. All rights reserved.
//
//	Future features:
//	* Expand/contract world in response to user input?
//	* Device rotation?  (During zooms?)
//	* Flash scroll indicators?
//	* True min/max zoom?
//	* Scroll-to-top override?
//	* Initial position/scale?
//	* Resizing of the scroll view?
//	* Text?
//	* Animations?
//
//	Spaces:
//	* Model space (The basic space in which a model's shape is defined)
//	* World space (Model spaces are scaled and rotated, then shifted to overlay world space)
//	* View space (World space is scaled to overlay view space)
//		. View space is identical with the content view's client space
//	* Tiled space (View space is scaled to overlay tiled space)
//		. Used to determine visible tiles
//	* Tile space (View space is shifted to overlay a particular tile space)
//		. A tile space is identical with a tile's client space
//  * Content space (Content view/view space is scaled and shifted to overlay content space)
//		. Content space is identical with the scroll view's client space
//	* Screen space (Content space is subjected to the normal view->screen transforms)
//
//	Older fix:
//	An earlier version of this code omitted the content view, and tiled the content space
//	directly.  During zooming, a special tile (4x the normal area) was provided for zooming.
//	Since only a subset of view space was being zoomed, the content offset had to be
//	adjusted after the zoom completed.  This would have worked, but for the fact that the
//	event system often delivered scroll events near the original content offset *after* the
//	content offset had been updated.  This moved the eye point wildly out of position.  To
//	fix this, the content offset was not updated immediately; instead, a "shift" factor was
//	included in the tile matrix calculations, which effectively moved the eye point from its
//	old position to its new one, and an asychronous call was scheduled to perform this shift
//	permanently, once the system was know to be quiescent.
//
//	To calculate the shift, it was assumed that:
//	* The origin of the zoom tile can be converted to world space at the original scale
//		. cp = wp*os + mr
//	* The origin->content offset vector can be converted to world space at the new scale
//		. wv*ns = co - cp
//	Therefore, 
//		. sh = co - ((wp + wv)*ns + mr)
//		. sh = co - ((cp - mr)/os + wv)*ns - mr
//		. sh = co - (cp - mr)*ns/os - wv*ns - mr
//		. sh = co - cp*rs + mr*rs - co + cp - mr
//		. sh = co - co + cp - cp*rs + mr*rs - mr
//		. sh = (1 - rs)*cp + (rs - 1)*mr
//	* Symbol key:
//		. cp: content space position of tile origin
//		. wp: world space position of tile origin
//		. os: original scale
//		. ns: new scale
//		. rs: relative scale (ns = os*rs)
//		. mr: margin (early code included a margin in the world->content xform)
//		. co: content offset (after zoom)
//		. wv: world space vector from tile origin to content offset
//		. sh: shift for tile transform; e.g. MakeTranslate(shift.x+...,shift.y+...)
//

#import "Path.h"
#import "PathGroup.h"
#import "FieldViewController.h"


@interface FieldViewController ()

@property (nonatomic, retain) UIScrollView* scrollView;

- (void)sizeContent;
- (void)handleGesture:(UIGestureRecognizer*)gestureRecognizer;

@end


@implementation FieldViewController

@synthesize content;

- (UIScrollView*)scrollView
{
	return (UIScrollView*) self.view;
}

- (void)setScrollView:(UIScrollView*)newScrollView
{
	self.view = newScrollView;
}


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];

	world = CGSizeMake(320*50, 460*25);
	scale = 1;
	
	UITapGestureRecognizer* gr = [[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleGesture:)] autorelease];
	[content addGestureRecognizer:gr];
	
	[self sizeContent];
	self.scrollView.contentInset = UIEdgeInsetsMake(25, 50, 25, 50);
	self.scrollView.contentOffset = CGPointMake(0, 0);
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc
{
	// Clear weak delegate reference
	if (self.scrollView.delegate == self)
	{
		self.scrollView.delegate = nil;
	}
	
	[content release];
    [super dealloc];
}

#pragma mark Scroll view delegate methods

- (BOOL)scrollViewShouldScrollToTop:(UIScrollView*)scrollView
{
	return YES;
}


// I've turned off zooming, for two reasons:
//  a.) Multithread considerations arise when using a CATiledLayer
//  b.) The naive implementation I had hacked together didn't look very good
//
// Simply turning on this code won't make zooming work; the logic to connect
// this object's scale member to the TiledView's world->view transform is
// missing.
- (UIView*)viewForZoomingInScrollView:(UIScrollView*)scrollView
{
	return content;
}


- (void)scrollViewDidEndZooming:(UIScrollView*)scrollView withView:(UIView*)view atScale:(float)relScale
{
	return;
	// Cache offset before zoom scale is reset
	CGPoint offset = self.scrollView.contentOffset;
	
	// Transfer zoom from scroll view to world->view transform; resize content at the new scale
	self.scrollView.zoomScale = 1.0;
	scale *= relScale;
	[self sizeContent];
	
	// Restore the old offset
	self.scrollView.contentOffset = offset;
}

#pragma mark Extension methods

- (void)sizeContent
{
	self.content.frame = CGRectMake(0, 0, world.width*scale, world.height*scale);
	self.scrollView.contentSize = CGSizeMake(CGRectGetMaxX(self.content.frame),
											 CGRectGetMaxY(self.content.frame));

	CGSize vpSize = self.scrollView.frame.size;
	CGFloat zoomOutLevels = MAX(ceil(log2(MAX(world.width*scale/vpSize.width, world.height*scale/vpSize.height))), 0);
	CGFloat zoomInLevels = 10;

	[(CATiledLayer*)self.content.layer setLevelsOfDetail:zoomOutLevels+zoomInLevels+1];
	[(CATiledLayer*)self.content.layer setLevelsOfDetailBias:zoomInLevels];

	self.scrollView.minimumZoomScale = pow(2, -zoomOutLevels);
	self.scrollView.maximumZoomScale = pow(2, zoomInLevels);
	
	[self.content.layer setNeedsDisplay];
}


- (void)handleGesture:(UIGestureRecognizer*)gestureRecognizer
{
	CGPoint p = [gestureRecognizer locationInView:content];
	NSLog(@"Tap Point: %.0f, %.0f",p.x,p.y);
}

@end
